
using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using ClarionSimulation;
using WorldServerLibrary;
using WorldServerLibrary.Model;
using WorldServerLibrary.Exceptions;

namespace ClarionDEMO
{
	class MainClass
	{
		#region properties
		private WorldServer worldServer = null;
        private ClarionAgent agent;
        String creatureId = String.Empty;
        String creatureName = String.Empty;
		Thing closestObject = null;
		Thing jewel = null;
		double prad = 0;
		#endregion

		#region constructor
		public MainClass() {
			Console.WriteLine ("Clarion Demo V0.4");
			agent = new ClarionAgent();
            agent.OnNewVisualSensorialInformation += new InputVisualSensorialInformationEventHandler(agent_OnNewVisualSensorialInformation);
            agent.OnNewExternalActionSelected += new OutputActionChunkEventHandler(agent_OnNewExternalActionSelected);
			try
            {
                worldServer = new WorldServer("localhost", 4011);
                String message = worldServer.Connect();

                if (worldServer != null && worldServer.IsConnected)
                {
                    Console.Out.WriteLine ("[SUCCESS] " + message + "\n");
					worldServer.SendWorldReset();
                    worldServer.NewCreature(400, 200, 0, out creatureId, out creatureName);
					worldServer.SendCreateLeaflet();
                    worldServer.NewBrick(4, 747, 2, 800, 567);
                    worldServer.NewBrick(4, 50, -4, 747, 47);
                    worldServer.NewBrick(4, 49, 562, 796, 599);
                    worldServer.NewBrick(4, -2, 6, 50, 599);

					// IA006 Criar joias e comidas
					Random r = new Random(DateTime.Now.Millisecond);
					int pos_x = 0;
					int pos_y = 0;

					// Joias de cada cor
					for(int j = 0; j < 6; j++)
					{
						for(int i = 0; i < 5; i++)
						{
							pos_x = (int)r.Next(100, 700);
							pos_y = (int)r.Next(100, 500);

							worldServer.NewJewel(j, pos_x, pos_y);
						}
					}

					// Comida
					for(int i = 0; i < 3; i++)
					{
						pos_x = (int)r.Next(100, 700);
						pos_y = (int)r.Next(100, 500);

						worldServer.NewFood(0, pos_x, pos_y);
					}

                    if (!String.IsNullOrWhiteSpace(creatureId))
                    {
                        worldServer.SendStartCamera(creatureId);
                        worldServer.SendStartCreature(creatureId);
                    }

                    Console.Out.WriteLine("Creature created with name: " + creatureId + "\n");
                    agent.Run();
					Console.Out.WriteLine("Running Simulation ...\n");
                }
            }
            catch (WorldServerInvalidArgument invalidArtgument)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Invalid Argument: {0}\n", invalidArtgument.Message));
            }
            catch (WorldServerConnectionError serverError)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Is is not possible to connect to server: {0}\n", serverError.Message));
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Unknown Error: {0}\n", ex.Message));
            }
		}
		#endregion

		#region Methods
		public static void Main (string[] args)	{
			new MainClass();
		}

        IList<Thing> agent_OnNewVisualSensorialInformation()
        {
            IList<Thing> response = null;

            if (worldServer != null && worldServer.IsConnected)
            {

                response = worldServer.SendGetCreatureState(creatureName);

				Creature creature = (Creature) response.Where(item => (item.CategoryId == Thing.CATEGORY_CREATURE)).First();
				
				//IA006
				//if (response.Any (item => (item.CategoryId == Thing.CATEGORY_JEWEL))) {
					//jewel = response.OrderBy (item => item.DistanceToCreature).First ();
					//jewel = response.Where(item => item.CategoryId == Thing.CATEGORY_JEWEL).OrderBy(item => item.DistanceToCreature).First();
					//Console.Out.WriteLine (" Closest jewel: " + jewel.Name + " (" + jewel.Material.Color + "); reward: " + jewel.Reward + "\n");
				//}
				
				IEnumerable<Thing> jewelSorted = response.Where(item => (item.CategoryId == Thing.CATEGORY_JEWEL && creature.inLeaflet(item.Material.Color)));
				if (jewelSorted.Any ()) {
					jewel = jewelSorted.OrderBy (item => item.DistanceToCreature).First ();
				}

				IEnumerable<Thing> sorted = response.Where(item => (item.CategoryId != Thing.CATEGORY_CREATURE && item.CategoryId != Thing.CATEGORY_BRICK));
				if (sorted.Any ()) {
					closestObject = sorted.OrderBy (item => item.DistanceToCreature).First ();
				}

				prad = (Math.PI / 180) * response.First().Pitch;
			    while (prad > Math.PI) prad -= 2 * Math.PI;
			    while (prad < - Math.PI) prad += 2 * Math.PI;

            }

            return response;
        }

        void agent_OnNewExternalActionSelected(ClarionAgentActionType externalAction)
        {   Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
            if (worldServer != null && worldServer.IsConnected)
            {
                switch (externalAction)
                {
                    case ClarionAgentActionType.DO_NOTHING:
						Console.Out.WriteLine ("NOTHING");
                        // Do nothing as the own value says
                        break;
                    case ClarionAgentActionType.ROTATE_CLOCKWISE:
						Console.Out.WriteLine ("ROTATE");
					    double newprad = prad + Math.PI/1000;
					    while (newprad > Math.PI) newprad -= 2 * Math.PI;
			            while (newprad < - Math.PI) newprad += 2 * Math.PI;
                        worldServer.SendSetAngle(creatureId, 0, 0, newprad);
                        break;
                    case ClarionAgentActionType.GO_AHEAD:
						Console.Out.WriteLine ("GO_AHEAD");
                        worldServer.SendSetAngle(creatureId, 1, 1, prad);
                        break;
					//IA006
					case ClarionAgentActionType.MOVE_JEWEL:
						//Console.Out.WriteLine ("MOVE_JEWEL "+jewel.Name);
						worldServer.SendSetAngle (creatureId, 0, 0, prad);
						worldServer.SendSetGoTo (creatureId, 1, 1, jewel.X1, jewel.Y1);
						break;
					//IA006
					case ClarionAgentActionType.MOVE_FOOD:
						//Console.Out.WriteLine ("MOVE_FOOD");
						worldServer.SendSetAngle (creatureId, 0, 0, prad);
						worldServer.SendSetGoTo (creatureId, 1, 1, closestObject.X1, closestObject.Y1);
						break;
					//IA006
					case ClarionAgentActionType.GET:
						Console.Out.WriteLine ("GET");
						worldServer.SendSetAngle (creatureId, 0, 0, prad);
						worldServer.SendSackIt (creatureId, closestObject.Name);
						break;
					//IA006
					case ClarionAgentActionType.EAT:
						Console.Out.WriteLine ("EAT");
						worldServer.SendSetAngle (creatureId, 0, 0, prad);
						worldServer.SendEatIt (creatureId, closestObject.Name);
						break;



                    default:
                        break;
                }
            }
        }

        #endregion
	}
	
	
}
